/*
	AsciiToFloatCmd.cc

	Will Eighteen, 2005: will@opaline.demon.co.uk
*/

#include "AsciiToFloatCmd.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


AsciiToFloatCmd::AsciiToFloatCmd ( char *name, int active, char *txtFile, char *floatFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, txtFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, floatFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_percentDone = 0;
	_error = 0;
	_offset = 0;
	_scale = 1;
}


AsciiToFloatCmd::~AsciiToFloatCmd()
{	
	// empty
}
	

void AsciiToFloatCmd::doCommand()
{
	if ( _done == True )
		return;

	int percent, lineLength = 128;
	char text[5], textbuf [ lineLength ];
	int error = 0;

	if ( !_once )
	{
		// can we open source for read?
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		
		if ( in->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, in->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		// can we open dst for write?
		FileAccess *out = new FileAccess ( _dstFile, "w" );
		
		if ( out->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile write error: %s", _dstFile, out->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		if ( error )
		{
			delete in;
			delete out;
			_done = True;
			_error = error;
			return;
		}
		
		_in = in;
		_out = out;

		// element size for an ascii file depends on num digits per line
		_in->elementSize ( 1 );
		_in->numElements ( 1 );
		_out->elementSize ( sizeof ( float ) );
		_out->numElements ( 1 );
		_once = 1;
	}
	
	if ( _in->filePos() < _in->fileSize() )
	{
		_in->readF ( textbuf, lineLength );
		sscanf ( textbuf, "%f", &_fValue );
		_fValue = ( _fValue + _offset ) * _scale;
		_out->writeF ( &_fValue, sizeof ( float ), 1 );

		updateTimeout ( ( int ) ( _in->fileSize() / ( strlen ( textbuf ) + 1 ) ) );
		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}

	}
	else
	{
		delete _in;
		delete _out;
		_done = True;
	}
}
