/*
	IntRangeCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "IntRangeCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <InfoDialogManager.h>

#include "OpResources.h"	// this is the only resources file with global resources


IntRangeCmd::IntRangeCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void IntRangeCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[256];
	char help[320];
		
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&IntRangeCmd::enterCallback,
									&IntRangeCmd::cancelCallback, ( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%d,%d",mixwaveResources->minY(), mixwaveResources->maxY() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies integer value range for view: min,max; default %d,%d",
				mixwaveResources->minY(), mixwaveResources->maxY() );
	
	help [ sizeof ( help ) - 1 ] = 0;				
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void IntRangeCmd::undoCommand()
{
	// not implemented
}


void IntRangeCmd::enterCallback ( void *clientData, char *text )
{
	IntRangeCmd *obj = ( IntRangeCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->enter ( text );

	obj->_popupRealised = False;
	delete obj->_edit;
}

/* specify resource to be set on 'enter' or 'ok' */
void IntRangeCmd::enter ( char *text )
{
	// check for separator
	int sep = 0;
	
	for ( unsigned int count = 0; count < strlen ( text ); count++ )
		if ( count[text] == ',' )
			sep++;

	if ( sep < 1 )
	{
		char msg[300];
		snprintf ( msg, sizeof ( msg ), "%s invalid: missing comma separator.\n\
Format (int): minYvalue,maxYvalue", text );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	int minY, maxY;;

	sscanf ( text, "%d,%d", &minY, &maxY );

	if ( minY < maxY )
	{
		mixwaveResources->minY ( minY );
		mixwaveResources->maxY ( maxY );
	}
	else
	{
		mixwaveResources->minY ( -32768 );	// default to signed 16-bit
		mixwaveResources->maxY ( 32767 );
	}
}


void IntRangeCmd::cancelCallback ( void *clientData )
{
	IntRangeCmd *obj = ( IntRangeCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
