/*
    NewSpatialTemplateCmd.cc

    Will 18, 2001, 2006: will@opaline.demon.co.uk
*/


#include <DMenuCmd.h>
#include "OpResources.h"
#include "NewSpatialTemplateCmd.h"
#include "AppWindow.h"
#include <MessageDialogManager.h>
#include <InfoDialogManager.h>
#include "CreateSpatialTemplateCmd.h"
#include "FileAccess.h"
#include "CreateTemplateFileCmd.h"
#include "MixwaveUtils.h"
#include "GraphIntCmd.h"	/* for graphCmd->close() if required */


NewSpatialTemplateCmd::NewSpatialTemplateCmd ( char *name, int active,
												NewSpatialTemplateCallback callback, void *clientData ) : NoUndoCmd ( name, active )
{
	_menuRealised = False;
	_name = name;
	_callback = callback;
	_clientData = clientData;
	_pakData[0] = 0;
	_baseName[0] = 0;
	_error[0] = 0;
	_spatialPak = NULL;
	_waveNum = 0;
	_defaultTemplateSize =  0;
}


void NewSpatialTemplateCmd::doCommand()
{
	if ( _menuRealised == True )
		return;

	_spatialPak = new SpatialPak;

	unsigned int n;
	unsigned int maxNameLength = 0;

	if ( (*wavePak)[0].pWave->first() == NULL )
		return;

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;

	if ( maxNameLength < 5 )
		maxNameLength = 5;	// accomodate 'quit' at a minimum

	_arrayItems++;

	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( (char *) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Quit" );

	_menu = new DMenuCmd ( "Wave menu", TRUE, TRUE,
				&NewSpatialTemplateCmd::menuSelectionCallback,
				( void * ) this, "Current wave files",
				_arrayItems, maxNameLength, ( char * ) &strings );

	_menuRealised = True;
	_menu->execute();
}


void NewSpatialTemplateCmd::menuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;	// dummy obj

	if ( ( strcmp ( label, "Quit" ) == 0 ) || ( selectedItemNum == 0 ) )
	{
		delete obj->_spatialPak;
		obj->_menuRealised = False;
	}
	else
	{
		int count, found;

		// scan base list for returned name pak index
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			found = 0;

			if ( strcmp ( label, (*wavePak)[count].pWave->name() ) == 0 )
			{
				obj->waveSelected ( label, (*wavePak)[count].pWave->waveNum() );
				found = 1;
				count = wavePak->Count();
			}
		}

		if ( !found && mixwaveResources->enableMessagePopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	delete obj->_menu;
}


void NewSpatialTemplateCmd::waveSelected ( char *filename, unsigned int waveNum )
{
	MixwaveUtils *utils = new MixwaveUtils;
	int error = 0;

	if ( (*wavePak)[waveNum-1].graphCmd != NULL )	/* wave view realised */
	{
		(*wavePak)[waveNum-1].graphCmd->close();

		if ( mixwaveResources->enableWarningPopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s view closed for\n\
spatial template creation.", utils->fileFromPath( (*wavePak)[waveNum-1].pWave->name() ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
		}
	}

	if ( (*wavePak)[waveNum-1].fLock == ACTIVE )
	{
		snprintf ( _error, sizeof ( _error ), "%s\nWave already in use!.",
					(*wavePak)[waveNum-1].pWave->name() );

		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}
	else
		(*wavePak)[waveNum-1].fLock = ACTIVE;

	if ( (*wavePak)[waveNum-1].pWave->numChannels() != 1 )
	{
		snprintf ( _error, sizeof ( _error ), "%s\nchannels = %u: only mono files can be viewed.",
					(*wavePak)[waveNum-1].pWave->name(),
					(*wavePak)[waveNum-1].pWave->numChannels() );

		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}

	if ( (*wavePak)[waveNum-1].pWave->dataLength() == 0 )
	{
		snprintf ( _error, sizeof ( _error ), "%s\nNo data in file (use 'Edit/Copy' to insert data).",
					(*wavePak)[waveNum-1].pWave->name() );
		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}		

	unsigned int baseLength, bytesPerSample;
	bytesPerSample = (*wavePak)[waveNum-1].pWave->bitsPerSample() / 8;

	if ( strlen ( filename ) > ( sizeof ( _baseName ) - 1 ) )
	{
		snprintf ( _error, sizeof ( _error ), "NewSpatialTemplateCmd error: baseName %d - insufficient\
 space for name\n%s.", sizeof ( _baseName ), filename );
		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}

	char waveName [ strlen ( utils->fileFromPath ( filename ) ) + 1 ];
	strncpy ( waveName, utils->fileFromPath ( filename ), sizeof ( waveName ) );
	waveName [ sizeof ( waveName ) - 1 ] = 0;

	baseLength = strstr ( waveName, ".wav" ) - waveName;

	if ( baseLength > ( sizeof ( _baseName ) - 1 ) )
		baseLength = sizeof ( _baseName ) - 1;

	strncpy ( _baseName, waveName , baseLength );
	_baseName [ baseLength ] = 0;
	strcpy ( waveName, _baseName );

	strncpy ( _baseName, mixwaveResources->workingDir(), sizeof ( _baseName ) );
	_baseName [ sizeof ( _baseName ) - 1 ] = 0;

	strncat ( _baseName, waveName, sizeof ( _baseName ) - strlen ( _baseName ) - 1 );
	_baseName [ sizeof ( _baseName) - 1 ] = 0;

	strncpy ( _pakData, _baseName, sizeof ( _pakData ) );
	_pakData [ sizeof ( _pakData ) - 1 ] = 0;

	if ( ( strlen ( _pakData ) + strlen ( mixwaveResources->cspatialScoreSuffix() ) ) < sizeof ( _pakData ) )
	{
		strncat ( _pakData, mixwaveResources->cspatialScoreSuffix(), sizeof ( _pakData ) - strlen ( _pakData ) - 1 );
		_pakData [ sizeof ( _pakData ) - 1 ] = 0;
		_waveNum = waveNum;
	}
	else
	{
		snprintf ( _error, sizeof ( _error ), "%s\nInsufficient BUFSIZE (%d) for name\nin\
NewSpatialTemplateCmd.h.", _pakData, BUFSIZE );
		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}

	_defaultTemplateSize = mixwaveResources->defaultTemplateSize();

	// csound imposes limit on file size, about 6 minutes - set in OpResources.cc
	if ( _defaultTemplateSize > ( mixwaveResources->maxGenData() / bytesPerSample ) )
	{
		char string1[12];
		char string2[12];

		strncpy ( string1, utils->sampleNumToTimeString (
					( mixwaveResources->maxGenData() / bytesPerSample ),
					(*wavePak)[waveNum-1].pWave->sampleRate(),
					sizeof ( string1 ), mixwaveResources->fpDigits() ), sizeof ( string1 ) );
		string1 [ sizeof ( string1 ) - 1 ] =  0;

		strncpy ( string2, utils->sampleNumToTimeString ( (*wavePak)[_waveNum-1].pWave->totalSamples(),
					(*wavePak)[waveNum-1].pWave->sampleRate(), sizeof ( string2 ),
				  	mixwaveResources->fpDigits() ), sizeof ( string2 ) );
		string2 [ sizeof ( string2 ) - 1 ] = 0;

		snprintf ( _error, sizeof ( _error ), "Maximum spatial template length \
exceeded. Maximum\nis %d samples (%s minutes).\n%s:\nduration is %s \
minutes:seconds.", mixwaveResources->maxGenData() / bytesPerSample, string1,
(*wavePak)[_waveNum-1].pWave->name(), string2 );
		_error [ sizeof ( _error ) - 1 ] = 0;
		error++;
	}

	if ( error )
	{
		messageDialogManager->post ( _error );
		(*wavePak)[waveNum-1].fLock = INACTIVE;
		_pakData[0] = 0;
		_baseName[0] = 0;
		_waveNum = 0;
		delete _spatialPak;
		_menuRealised = False;
		delete utils;
		return;
	}

	if ( _defaultTemplateSize > (*wavePak)[waveNum-1].pWave->totalSamples() )
		_defaultTemplateSize = (*wavePak)[waveNum-1].pWave->totalSamples();

	pakInitialise();	// requires base wave number

	if ( mixwaveResources->enableWarningPopups() == True )
	{
		char waveFile [ strlen ( _baseName ) + 14 ];

		strncpy ( waveFile, _baseName, sizeof ( waveFile ) );
		waveFile [ sizeof ( waveFile ) - 1 ] = 0;
		strncat ( waveFile, "_pakData", sizeof ( waveFile ) - strlen ( waveFile ) - 1 );
		waveFile [ sizeof ( waveFile ) - 1 ] = 0;

		FileAccess *target = new FileAccess ( waveFile, "r" );
		error = target->errNum();
		delete target;

		if ( error == 0 )       // succeeded, file already exists
		{

		char msg[BUFSIZE];
			char name[BUFSIZE];
			strncpy ( name, utils->fileFromPath ( filename ), sizeof ( name ) );
			snprintf ( msg, sizeof ( msg ), "Spatial template for %s\n\
already exists: continue?", name );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg, ( void * ) this,
							&NewSpatialTemplateCmd::overwriteOkCallback,
							&NewSpatialTemplateCmd::overwriteCancelCallback );
		}
		else
			waveSelected();
	}
	else
		waveSelected();

	_menuRealised = False;
	delete utils;
}


void NewSpatialTemplateCmd::overwriteOkCallback ( void *clientData )
{
        NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

        obj->waveSelected();
}


void NewSpatialTemplateCmd::overwriteCancelCallback ( void *clientData )
{
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	obj->overwriteCancelled();
}


void NewSpatialTemplateCmd::overwriteCancelled()
{
	_pakData[0] = 0;
	_menuRealised = False;
	delete _spatialPak;
	(*wavePak)[_waveNum-1].fLock = INACTIVE;
}


void NewSpatialTemplateCmd::waveSelected()
{
	// start array handling process; either read or check data arrays
	// terminating checkSpatialTemplateCmd object instantiation for views
	if ( _waveNum != 0 )
	{
		if ( (*wavePak)[_waveNum-1].pWave->sampleRate() != mixwaveResources->samplesPerSec() )
		{
			char msg[BUFSIZE<<1];
			snprintf ( msg, sizeof ( msg ), "Wave sample rate %d conflicts with\n\
resource sample rate %d\nUse Options/File/Wave parameters to set\n\
this resource (parameter 2) to the wave sample rate.",
			(*wavePak)[_waveNum-1].pWave->sampleRate(), mixwaveResources->samplesPerSec() );
			msg [ sizeof ( msg ) -1 ] = 0;
			messageDialogManager->post ( msg );
			(*wavePak)[_waveNum-1].fLock = INACTIVE;
			return;
		}

		strncpy ( _spatialPak->name, _baseName, sizeof ( _spatialPak->name ) - 1 );
		_spatialPak->name [ sizeof ( _spatialPak->name ) - 1 ] = 0;

		checkReverbTemplateFile();	// chain creation process for arrays
	}
}


void NewSpatialTemplateCmd::checkReverbTemplateFile()
{
	char filename [ strlen ( _baseName ) + 12 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_reverb.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;
	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkReverbTemplateCmd = new CreateTemplateFileCmd (
										"CreateReverbTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkReverbTemplateCmd->execute ( &NewSpatialTemplateCmd::checkReverbTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		// call reverb template finished with null ptr to skip file creation check
		checkReverbTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkReverbTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *rvbObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateReverbTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkReverbTemplateFinished ( rvbObj, "" );

	delete rvbObj;
}


void NewSpatialTemplateCmd::checkReverbTemplateFinished ( CreateTemplateFileCmd *rvbObj, char *filename )
{
	if ( rvbObj != NULL )
	{
		if ( rvbObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check reverb template \
file failed.", rvbObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkRangeTemplateFile();	// range of sound source from observer
}


void NewSpatialTemplateCmd::checkRangeTemplateFile()
{
	char filename [ strlen ( _baseName ) + 11 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_range.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkRangeTemplateCmd = new CreateTemplateFileCmd (
										"CreateRangeTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkRangeTemplateCmd->setDefaultValue ( 1 );	// arbitrary 'unit' distance
		checkRangeTemplateCmd->execute ( &NewSpatialTemplateCmd::checkRangeTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkRangeTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkRangeTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *rngObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateRangeTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkRangeTemplateFinished ( rngObj, "" );

	delete rngObj;
}


void NewSpatialTemplateCmd::checkRangeTemplateFinished ( CreateTemplateFileCmd *rngObj, char *filename )
{
	if ( rngObj != NULL )
	{
		if ( rngObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check reverb template \
file failed.", rngObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkBwidthTemplateFile();
}


void NewSpatialTemplateCmd::checkBwidthTemplateFile()
{
	char filename [ strlen ( _baseName ) + 12 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_bwidth.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkBwidthTemplateCmd = new CreateTemplateFileCmd (
										"CreateBwidthTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkBwidthTemplateCmd->execute ( &NewSpatialTemplateCmd::checkBwidthTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkBwidthTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkBwidthTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *bwObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateBwidthTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkBwidthTemplateFinished ( bwObj, "" );

	delete bwObj;
}


void NewSpatialTemplateCmd::checkBwidthTemplateFinished ( CreateTemplateFileCmd *bwObj, char *filename )
{
	if ( bwObj != NULL )
	{
		if ( bwObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check bandwidth template \
file failed.", bwObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkSweepTemplateFile();
}


void NewSpatialTemplateCmd::checkSweepTemplateFile()
{
	char filename [ strlen ( _baseName ) + 11 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_sweep.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkSweepTemplateCmd = new CreateTemplateFileCmd (
										"CreateSweepTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkSweepTemplateCmd->execute ( &NewSpatialTemplateCmd::checkSweepTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkSweepTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkSweepTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *swObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateSweepTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkSweepTemplateFinished ( swObj, "" );

	delete swObj;
}


void NewSpatialTemplateCmd::checkSweepTemplateFinished ( CreateTemplateFileCmd *swObj, char *filename )
{
	if ( swObj != NULL )
	{
		if ( swObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check sweep template \
file failed.", swObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkPanTemplateFile();
}


void NewSpatialTemplateCmd::checkPanTemplateFile()
{
	char filename [ strlen ( _baseName ) + 9 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_pan.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkPanTemplateCmd = new CreateTemplateFileCmd (
										"CreatePanTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkPanTemplateCmd->setDefaultValue ( 0.5 );	// pan central
		checkPanTemplateCmd->execute ( &NewSpatialTemplateCmd::checkPanTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkPanTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkPanTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *panObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreatePanTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkPanTemplateFinished ( panObj, "" );

	delete panObj;
}


void NewSpatialTemplateCmd::checkPanTemplateFinished ( CreateTemplateFileCmd *panObj, char *filename )
{
	if ( panObj != NULL )
	{
		if ( panObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check bandwidth template \
file failed.", panObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkDopplerTemplateFile();
}


void NewSpatialTemplateCmd::checkDopplerTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_dplr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkDopplerTemplateCmd = new CreateTemplateFileCmd (
										"CreateDopplerTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkDopplerTemplateCmd->setDefaultValue ( 1 );	// 'no effect'
		checkDopplerTemplateCmd->execute ( &NewSpatialTemplateCmd::checkDopplerTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkDopplerTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkDopplerTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *dplrObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateDopplerTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkDopplerTemplateFinished ( dplrObj, "" );

	delete dplrObj;
}


void NewSpatialTemplateCmd::checkDopplerTemplateFinished ( CreateTemplateFileCmd *dplrObj, char *filename )
{
	if ( dplrObj != NULL )
	{
		if ( dplrObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check doppler pitch \
shift template file failed.", dplrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkReverbLTemplateFile();	// now commence data files for left and right reflection parameters
}


void NewSpatialTemplateCmd::checkReverbLTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_rvbl.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkReverbLTemplateCmd = new CreateTemplateFileCmd (
										"CreateReverbLTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkReverbLTemplateCmd->execute ( &NewSpatialTemplateCmd::checkReverbLTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkReverbLTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkReverbLTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *rvblObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateReverbLTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkReverbLTemplateFinished ( rvblObj, "" );

	delete rvblObj;
}


void NewSpatialTemplateCmd::checkReverbLTemplateFinished ( CreateTemplateFileCmd *rvblObj, char *filename )
{
	if ( rvblObj != NULL )
	{
		if ( rvblObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check LH reverb template \
file failed.", rvblObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkReverbRTemplateFile();
}


void NewSpatialTemplateCmd::checkReverbRTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_rvbr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkReverbRTemplateCmd = new CreateTemplateFileCmd (
										"CreateReverbRTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkReverbRTemplateCmd->execute ( &NewSpatialTemplateCmd::checkReverbRTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists

		// call reverb template finished with null ptr to skip file creation check
		checkReverbRTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkReverbRTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *rvbrObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateReverbRTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkReverbRTemplateFinished ( rvbrObj, "" );

	delete rvbrObj;
}


void NewSpatialTemplateCmd::checkReverbRTemplateFinished ( CreateTemplateFileCmd *rvbrObj, char *filename )
{
	if ( rvbrObj != NULL )
	{
		if ( rvbrObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check RH reverb template \
file failed.", rvbrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkPanLTemplateFile();
}


void NewSpatialTemplateCmd::checkPanLTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_panl.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkPanLTemplateCmd = new CreateTemplateFileCmd (
										"CreatePanLTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkPanLTemplateCmd->setDefaultValue ( 0.5 ); 	// pan central
		checkPanLTemplateCmd->execute ( &NewSpatialTemplateCmd::checkPanLTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkPanLTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkPanLTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *panlObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreatePanLTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkPanLTemplateFinished ( panlObj, "" );

	delete panlObj;
}


void NewSpatialTemplateCmd::checkPanLTemplateFinished ( CreateTemplateFileCmd *panlObj, char *filename )
{
	if ( panlObj != NULL )
	{
		if ( panlObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check LH pan template \
file failed.", panlObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkPanRTemplateFile();
}


void NewSpatialTemplateCmd::checkPanRTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_panr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkPanRTemplateCmd = new CreateTemplateFileCmd (
										"CreatePanRTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkPanRTemplateCmd->setDefaultValue ( 0.5 );	// pan central
		checkPanRTemplateCmd->execute ( &NewSpatialTemplateCmd::checkPanRTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkPanRTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkPanRTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *panrObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreatePanRTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkPanRTemplateFinished ( panrObj, "" );

	delete panrObj;
}


void NewSpatialTemplateCmd::checkPanRTemplateFinished ( CreateTemplateFileCmd *panrObj, char *filename )
{
	if ( panrObj != NULL )
	{
		if ( panrObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check RH pan template \
file failed.", panrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkBwidthLTemplateFile();
}


void NewSpatialTemplateCmd::checkBwidthLTemplateFile()
{
	char filename [ strlen ( _baseName ) + 9 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_bwl.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkBwidthLTemplateCmd = new CreateTemplateFileCmd (
										"CreateBwidthLTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkBwidthLTemplateCmd->execute ( &NewSpatialTemplateCmd::checkBwidthLTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkBwidthLTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkBwidthLTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *bwlObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateBwidthLTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkBwidthLTemplateFinished ( bwlObj, "" );

	delete bwlObj;
}


void NewSpatialTemplateCmd::checkBwidthLTemplateFinished ( CreateTemplateFileCmd *bwlObj, char *filename )
{
	if ( bwlObj != NULL )
	{
		if ( bwlObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check LH filter \
bandwidth template file failed.", bwlObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkBwidthRTemplateFile();
}

void NewSpatialTemplateCmd::checkBwidthRTemplateFile()
{
	char filename [ strlen ( _baseName ) + 9 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_bwr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkBwidthRTemplateCmd = new CreateTemplateFileCmd (
										"CreateBwidthRTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkBwidthRTemplateCmd->execute ( &NewSpatialTemplateCmd::checkBwidthRTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkBwidthRTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkBwidthRTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *bwrObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateBwidthRTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkBwidthRTemplateFinished ( bwrObj, "" );

	delete bwrObj;
}


void NewSpatialTemplateCmd::checkBwidthRTemplateFinished ( CreateTemplateFileCmd *bwrObj, char *filename )
{
	if ( bwrObj != NULL )
	{
		if ( bwrObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check RH filter \
bandwidth template file failed.", bwrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}
	}

	checkSweepLTemplateFile();
}


void NewSpatialTemplateCmd::checkSweepLTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_swpl.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkSweepLTemplateCmd = new CreateTemplateFileCmd (
										"CreateSweepLTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkSweepLTemplateCmd->execute ( &NewSpatialTemplateCmd::checkSweepLTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkSweepLTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkSweepLTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *swlObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateSweepLTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkSweepLTemplateFinished ( swlObj, "" );

	delete swlObj;
}


void NewSpatialTemplateCmd::checkSweepLTemplateFinished ( CreateTemplateFileCmd *swlObj, char *filename )
{
	if ( swlObj != NULL )
	{
		if ( swlObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check LH filter \
sweep freq. template file failed.", swlObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	checkSweepRTemplateFile();
}


void NewSpatialTemplateCmd::checkSweepRTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_swpr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkSweepRTemplateCmd = new CreateTemplateFileCmd (
										"CreateSweepRTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkSweepRTemplateCmd->execute ( &NewSpatialTemplateCmd::checkSweepRTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkSweepRTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkSweepRTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *swrObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateSweepRTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkSweepRTemplateFinished ( swrObj, "" );

	delete swrObj;
}


void NewSpatialTemplateCmd::checkSweepRTemplateFinished ( CreateTemplateFileCmd *swrObj, char *filename )
{
	if ( swrObj != NULL )
	{
		if ( swrObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check RH filter \
sweep freq. template file failed.", swrObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	checkAmpLTemplateFile();
}


void NewSpatialTemplateCmd::checkAmpLTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_ampl.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkAmpLTemplateCmd = new CreateTemplateFileCmd (
										"CreateAmpLTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkAmpLTemplateCmd->setDefaultValue ( 1 );	// unity gain
		checkAmpLTemplateCmd->execute ( &NewSpatialTemplateCmd::checkAmpLTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkAmpLTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkAmpLTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *amplObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateAmpLTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkAmpLTemplateFinished ( amplObj, "" );

	delete amplObj;
}


void NewSpatialTemplateCmd::checkAmpLTemplateFinished ( CreateTemplateFileCmd *amplObj, char *filename )
{
	if ( amplObj != NULL )
	{
		if ( amplObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check LH amplitude \
template file failed.", amplObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	checkAmpRTemplateFile();
}


void NewSpatialTemplateCmd::checkAmpRTemplateFile()
{
	char filename [ strlen ( _baseName ) + 10 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_ampr.flt", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() != 0  )
	{	// if not read at a minimum, check a new file
		delete in;
		unsigned int dataType = 1;	// float

		CreateTemplateFileCmd *checkAmpRTemplateCmd = new CreateTemplateFileCmd (
										"CreateAmpRTemplate", TRUE, filename,
										_defaultTemplateSize,
										mixwaveResources->blockLength(), dataType );

		checkAmpRTemplateCmd->setDefaultValue ( 1 );	// unity gain
		checkAmpRTemplateCmd->execute ( &NewSpatialTemplateCmd::checkAmpRTemplateFinishedCallback, ( void * ) this );
	}
	else
	{
		delete in;	// file already exists
		checkAmpRTemplateFinished ( ( CreateTemplateFileCmd * ) NULL, filename );
	}
}


void NewSpatialTemplateCmd::checkAmpRTemplateFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateTemplateFileCmd *amprObj = ( CreateTemplateFileCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		messageDialogManager->post ( "CreateAmpRTemplateInterrupted!" );
		obj->_menuRealised = False;
		(*wavePak)[obj->_waveNum-1].fLock = INACTIVE;
	}
	else
		obj->checkAmpRTemplateFinished ( amprObj, "" );

	delete amprObj;
}


void NewSpatialTemplateCmd::checkAmpRTemplateFinished ( CreateTemplateFileCmd *amprObj, char *filename )
{
	if ( amprObj != NULL )
	{
		if ( amprObj->error() != 0 )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "%s\nError: check RH amplitude \
template file failed.", amprObj->target() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}

	checkSpatialPakFile();
}


void NewSpatialTemplateCmd::checkSpatialPakFile()
{
	char filename [ strlen ( _baseName ) + 9 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_pakData", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix
	filename [ sizeof ( filename ) - 1 ] = 0;

	FileAccess *in = new FileAccess ( filename, "r" );

	if ( in->errNum() == 0  )
	{
		// use file-based values if possible
		in->numElements ( 1 );
		in->elementSize ( sizeof ( SpatialPak ) );
		in->readF ( _spatialPak );
	}

	delete in;	// close file whether read or checkd
	// last link in chain is execute views to modify files, etc. Terminated on wave
	// cancel or close
	CreateSpatialTemplateCmd *cstCmd = new CreateSpatialTemplateCmd ( ( char * ) _baseName, TRUE, _waveNum, _spatialPak,
											&NewSpatialTemplateCmd::spatialTemplateCreatedCallback, ( void * ) this );
	cstCmd->execute();
}


void NewSpatialTemplateCmd::spatialTemplateCreatedCallback ( CreateSpatialTemplateCmd *cmd, void *clientData, unsigned int callbackReason )
{
	CreateSpatialTemplateCmd *cstObj = ( CreateSpatialTemplateCmd * ) cmd;
	NewSpatialTemplateCmd *obj = ( NewSpatialTemplateCmd * ) clientData;				

	obj->spatialTemplateCreated ( callbackReason );
	delete cstObj;
}


void NewSpatialTemplateCmd::spatialTemplateCreated ( unsigned int callbackReason )
{
	// this callback is triggered on wave view 'ok' and 'cancel' or external 'close'
	// indicating template creation process complete
	_callbackReason = callbackReason;

	char filename [ strlen ( _baseName ) + 9 ];
	strncpy ( filename, _baseName, sizeof ( filename ) );
	filename [ sizeof ( filename ) - 1 ] = 0;
	strncat ( filename, "_pakData", sizeof ( filename ) - strlen ( filename ) - 1 );	// s/b a resources-defined suffix

	if ( _callbackReason == 1 )	// 'ok'
	{
		FileAccess *out = new FileAccess ( filename, "w" );

		if ( out->errNum() != 0 )
			_callbackReason = 2;	// force 'template creation cancelled' in callback
									// overriding an 'ok' from wave view
		else
		{	// write new file - assume overwrite??
			// array and object ptrs nulled (reconstructed on re-entry)
			_spatialPak->rvbView = NULL;
			_spatialPak->rvbArray = NULL;
			_spatialPak->rvbFile = NULL;
			_spatialPak->rangeView = NULL;
			_spatialPak->rangeArray = NULL;
			_spatialPak->rangeFile = NULL;
			_spatialPak->bwView = NULL;
			_spatialPak->bwArray = NULL;
			_spatialPak->bwFile = NULL;
			_spatialPak->sweepView = NULL;
			_spatialPak->sweepArray = NULL;
			_spatialPak->sweepFile = NULL;
			_spatialPak->panView = NULL;
			_spatialPak->panArray = NULL;
			_spatialPak->panFile = NULL;
			_spatialPak->dopplerView = NULL;
			_spatialPak->dopplerArray = NULL;
			_spatialPak->dopplerFile = NULL;
			_spatialPak->rvbLView = NULL;
			_spatialPak->rvbLArray = NULL;
			_spatialPak->rvbLFile = NULL;
			_spatialPak->rvbRView = NULL;
			_spatialPak->rvbRArray = NULL;
			_spatialPak->rvbRFile = NULL;
			_spatialPak->panLView = NULL;
			_spatialPak->panLArray = NULL;
			_spatialPak->panLFile = NULL;
			_spatialPak->panRView = NULL;
			_spatialPak->panRArray = NULL;
			_spatialPak->panRFile = NULL;
			_spatialPak->bwLView = NULL;
			_spatialPak->bwLArray = NULL;
			_spatialPak->bwLFile = NULL;
			_spatialPak->bwRView = NULL;
			_spatialPak->bwRArray = NULL;
			_spatialPak->bwRFile = NULL;
			_spatialPak->sweepLView = NULL;
			_spatialPak->sweepLArray = NULL;
			_spatialPak->sweepLFile = NULL;
			_spatialPak->sweepRView = NULL;
			_spatialPak->sweepRArray = NULL;
			_spatialPak->sweepRFile = NULL;
			_spatialPak->ampLView = NULL;
			_spatialPak->ampLArray = NULL;
			_spatialPak->ampLFile = NULL;
			_spatialPak->ampRView = NULL;
			_spatialPak->ampRArray = NULL;
			_spatialPak->ampRFile = NULL;

			out->numElements ( 1 );
			out->elementSize ( sizeof ( SpatialPak ) );
			out->writeF ( ( void * ) _spatialPak);
		}

		delete out;
	}

	if ( _spatialPak )
		delete _spatialPak;

	(*wavePak)[_waveNum-1].fLock = INACTIVE;

	if ( _callback )
		_callback ( _clientData, _baseName, _callbackReason );

	_menuRealised = False;
	mixwaveResources->defaultTemplateSize ( _defaultTemplateSize );
}


void NewSpatialTemplateCmd::pakInitialise()
{	// initialise pak contents to sensible defaults - o/write by 'read struct if present'
	_spatialPak->name[0] = 0;
	_spatialPak->rvbView = NULL;
	_spatialPak->rvbArray = NULL;
	_spatialPak->rvbFile = NULL;
	_spatialPak->rvbStartRange = 0;
	_spatialPak->rvbEndRange = 0;
	_spatialPak->rvbStartSel = 0;
	_spatialPak->rvbEndSel = 0;
	_spatialPak->rvbXCurrent = 0;
	_spatialPak->rvbYCurrent = 0;
	_spatialPak->rvbFilePos = 0;
	_spatialPak->rvbArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->rvbDataOffset = 0;
	_spatialPak->rvbDataSize = 0;
	_spatialPak->rvbGraphNum = 0;
	_spatialPak->rvbMin = 0;
	_spatialPak->rvbMax = 1;	// reverb send = source signal 
	_spatialPak->rvbFLock = INACTIVE;

	_spatialPak->rangeView = NULL;
	_spatialPak->rangeArray = NULL;
	_spatialPak->rangeFile = NULL;
	_spatialPak->rangeStartRange = 0;
	_spatialPak->rangeEndRange = 0;
	_spatialPak->rangeStartSel = 0;
	_spatialPak->rangeEndSel = 0;
	_spatialPak->rangeXCurrent = 0;
	_spatialPak->rangeYCurrent = 0;
	_spatialPak->rangeFilePos = 0;
	_spatialPak->rangeArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->rangeDataOffset = 0;
	_spatialPak->rangeDataSize = 0;
	_spatialPak->rangeGraphNum = 0;
	_spatialPak->rangeMin = 1;	// 'unit' distance, effectively 'at ear'
	_spatialPak->rangeMax = 10;
	_spatialPak->rangeFLock = INACTIVE;

	_spatialPak->bwView = NULL;
	_spatialPak->bwArray = NULL;
	_spatialPak->bwFile = NULL;
	_spatialPak->bwStartRange = 0;
	_spatialPak->bwEndRange = 0;
	_spatialPak->bwStartSel = 0;
	_spatialPak->bwEndSel = 0;
	_spatialPak->bwXCurrent = 0;
	_spatialPak->bwYCurrent = 0;
	_spatialPak->bwFilePos = 0;
	_spatialPak->bwArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->bwDataOffset = 0;
	_spatialPak->bwDataSize = 0;
	_spatialPak->bwGraphNum = 0;
	_spatialPak->bwMin = 0;
	_spatialPak->bwMax = 11025;	// min must be <= 1/2 bandwidth to avoid aliasing (22050 for 16-bit wav)
	_spatialPak->bwFLock = INACTIVE;

	_spatialPak->sweepView = NULL;
	_spatialPak->sweepArray = NULL;
	_spatialPak->sweepFile = NULL;
	_spatialPak->sweepStartRange = 0;
	_spatialPak->sweepEndRange = 0;
	_spatialPak->sweepStartSel = 0;
	_spatialPak->sweepEndSel = 0;
	_spatialPak->sweepXCurrent = 0;
	_spatialPak->sweepYCurrent = 0;
	_spatialPak->sweepFilePos = 0;
	_spatialPak->sweepArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->sweepDataOffset = 0;
	_spatialPak->sweepDataSize = 0;
	_spatialPak->sweepGraphNum = 0;
	_spatialPak->sweepMin = 0;
	_spatialPak->sweepMax = 11025;	// sweepMax + bwidthMax <= sample rate/2
	_spatialPak->sweepFLock = INACTIVE;

	_spatialPak->panView = NULL;
	_spatialPak->panArray = NULL;
	_spatialPak->panFile = NULL;
	_spatialPak->panStartRange = 0;
	_spatialPak->panEndRange = 0;
	_spatialPak->panStartSel = 0;
	_spatialPak->panEndSel = 0;
	_spatialPak->panXCurrent = 0;
	_spatialPak->panYCurrent = 0;
	_spatialPak->panFilePos = 0;
	_spatialPak->panArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->panDataOffset = 0;
	_spatialPak->panDataSize = 0;
	_spatialPak->panGraphNum = 0;
	_spatialPak->panMin = 0;	// pan values won't need changing, full L to
	_spatialPak->panMax = 1;	// full R
	_spatialPak->panFLock = INACTIVE;

	_spatialPak->dopplerView = NULL;
	_spatialPak->dopplerArray = NULL;
	_spatialPak->dopplerFile = NULL;
	_spatialPak->dopplerStartRange = 0;
	_spatialPak->dopplerEndRange = 0;
	_spatialPak->dopplerStartSel = 0;
	_spatialPak->dopplerEndSel = 0;
	_spatialPak->dopplerXCurrent = 0;
	_spatialPak->dopplerYCurrent = 0;
	_spatialPak->dopplerFilePos = 0;
	_spatialPak->dopplerArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->dopplerDataOffset = 0;
	_spatialPak->dopplerDataSize = 0;
	_spatialPak->dopplerGraphNum = 0;
	_spatialPak->dopplerMin = 0;	// max doppler effect
	_spatialPak->dopplerMax = 1;	// doppler effect absent
	_spatialPak->dopplerFLock = INACTIVE;

	_spatialPak->panType = 0;
	_spatialPak->filterType= 0;
	_spatialPak->angularSep = 0.5;
	_spatialPak->signalScale = 1.0;
	_spatialPak->detune = 0.0;
	_spatialPak->phaser = 0; // off

	_spatialPak->rvbLView = NULL;
	_spatialPak->rvbLArray = NULL;
	_spatialPak->rvbLFile = NULL;
	_spatialPak->rvbLStartRange = 0 ;
	_spatialPak->rvbLEndRange = 0;
	_spatialPak->rvbLStartSel = 0;
	_spatialPak->rvbLEndSel = 0;
	_spatialPak->rvbLXCurrent = 0;
	_spatialPak->rvbLYCurrent = 0;
	_spatialPak->rvbLFilePos = 0;
	_spatialPak->rvbLArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->rvbLDataOffset = 0;
	_spatialPak->rvbLDataSize = 0;
	_spatialPak->rvbLGraphNum = 0;
	_spatialPak->rvbLMin = 0;
	_spatialPak->rvbLMax = 1;
	_spatialPak->rvbLFLock = INACTIVE;

	_spatialPak->rvbLView = NULL;
	_spatialPak->rvbRArray = NULL;
	_spatialPak->rvbRFile = NULL;
	_spatialPak->rvbRStartRange = 0;
	_spatialPak->rvbREndRange = 0;
	_spatialPak->rvbRStartSel = 0;
	_spatialPak->rvbREndSel = 0;
	_spatialPak->rvbRXCurrent = 0;
	_spatialPak->rvbRYCurrent = 0;
	_spatialPak->rvbRFilePos = 0;
	_spatialPak->rvbRArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->rvbRDataOffset = 0;
	_spatialPak->rvbRDataSize = 0;
	_spatialPak->rvbRGraphNum = 0;
	_spatialPak->rvbRMin = 0;
	_spatialPak->rvbRMax = 1;
	_spatialPak->rvbRFLock = INACTIVE;

	_spatialPak->panLView = NULL;
	_spatialPak->panLArray = NULL;
	_spatialPak->panLFile = NULL;
	_spatialPak->panLStartRange = 0;
	_spatialPak->panLEndRange = 0;
	_spatialPak->panLStartSel = 0;
	_spatialPak->panLEndSel = 0;
	_spatialPak->panLXCurrent = 0;
	_spatialPak->panLYCurrent = 0;
	_spatialPak->panLFilePos = 0;
	_spatialPak->panLArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->panLDataOffset = 0;
	_spatialPak->panLDataSize = 0;
	_spatialPak->panLGraphNum = 0;
	_spatialPak->panLMin = 0;
	_spatialPak->panLMax = 1;
	_spatialPak->panLFLock = INACTIVE;

	_spatialPak->panRView = NULL;
	_spatialPak->panRArray = NULL;
	_spatialPak->panRFile = NULL;
	_spatialPak->panRStartRange = 0;
	_spatialPak->panREndRange = 0;
	_spatialPak->panRStartSel = 0;
	_spatialPak->panREndSel = 0;
	_spatialPak->panRXCurrent = 0;
	_spatialPak->panRYCurrent = 0;
	_spatialPak->panRFilePos = 0;
	_spatialPak->panRArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->panRDataOffset = 0;
	_spatialPak->panRDataSize = 0;
	_spatialPak->panRGraphNum = 0;
	_spatialPak->panRMin = 0;
	_spatialPak->panRMax = 1;
	_spatialPak->panRFLock = INACTIVE;

	_spatialPak->panLType = 0;
	_spatialPak->panRType = 0;
	
	_spatialPak->bwLView = NULL;
	_spatialPak->bwLArray = NULL;
	_spatialPak->bwLFile = NULL;
	_spatialPak->bwLStartRange = 0;
	_spatialPak->bwLEndRange = 0;
	_spatialPak->bwLStartSel = 0;
	_spatialPak->bwLEndSel = 0;
	_spatialPak->bwLXCurrent = 0;
	_spatialPak->bwLYCurrent = 0;
	_spatialPak->bwLFilePos = 0;
	_spatialPak->bwLArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->bwLDataOffset = 0;
	_spatialPak->bwLDataSize = 0;
	_spatialPak->bwLGraphNum = 0;
	_spatialPak->bwLMin = 0;
	_spatialPak->bwLMax = 11025;
	_spatialPak->bwLFLock = INACTIVE;

	_spatialPak->bwRView = NULL;
	_spatialPak->bwRArray = NULL;
	_spatialPak->bwRFile = NULL;
	_spatialPak->bwRStartRange = 0;
	_spatialPak->bwREndRange = 0;
	_spatialPak->bwRStartSel = 0;
	_spatialPak->bwREndSel = 0;
	_spatialPak->bwRXCurrent = 0;
	_spatialPak->bwRYCurrent = 0;
	_spatialPak->bwRFilePos = 0;
	_spatialPak->bwRArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->bwRDataOffset = 0;
	_spatialPak->bwRDataSize = 0;
	_spatialPak->bwRGraphNum = 0;
	_spatialPak->bwRMin = 0;
	_spatialPak->bwRMax = 11025;
	_spatialPak->bwRFLock = INACTIVE;

	_spatialPak->sweepLView = NULL;
	_spatialPak->sweepLArray = NULL;
	_spatialPak->sweepLFile = NULL;
	_spatialPak->sweepLStartRange = 0;
	_spatialPak->sweepLEndRange = 0;
	_spatialPak->sweepLStartSel = 0;
	_spatialPak->sweepLEndSel = 0;
	_spatialPak->sweepLXCurrent = 0;
	_spatialPak->sweepLYCurrent = 0;
	_spatialPak->sweepLFilePos = 0;
	_spatialPak->sweepLArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->sweepLDataOffset = 0;
	_spatialPak->sweepLDataSize = 0;
	_spatialPak->sweepLGraphNum = 0;
	_spatialPak->sweepLMin = 0;
	_spatialPak->sweepLMax = 11025;
	_spatialPak->sweepLFLock = INACTIVE;

	_spatialPak->sweepRView = NULL;
	_spatialPak->sweepRArray = NULL;
	_spatialPak->sweepRFile = NULL;
	_spatialPak->sweepRStartRange = 0;
	_spatialPak->sweepREndRange = 0;
	_spatialPak->sweepRStartSel = 0;
	_spatialPak->sweepREndSel = 0;
	_spatialPak->sweepRXCurrent = 0;
	_spatialPak->sweepRYCurrent = 0;
	_spatialPak->sweepRFilePos = 0;
	_spatialPak->sweepRArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->sweepRDataOffset = 0;
	_spatialPak->sweepRDataSize = 0;
	_spatialPak->sweepRGraphNum = 0;
	_spatialPak->sweepRMin = 0;
	_spatialPak->sweepRMax = 11025;
	_spatialPak->sweepRFLock = INACTIVE;

	_spatialPak->delayL = 0;
	_spatialPak->delayR = 0;

	_spatialPak->ampLView = NULL;
	_spatialPak->ampLArray = NULL;
	_spatialPak->ampLFile = NULL;
	_spatialPak->ampLStartRange = 0;
	_spatialPak->ampLEndRange = 0;
	_spatialPak->ampLStartSel = 0;
	_spatialPak->ampLEndSel = 0;
	_spatialPak->ampLXCurrent = 0;
	_spatialPak->ampLYCurrent = 0;
	_spatialPak->ampLFilePos = 0;
	_spatialPak->ampLArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->ampLDataOffset = 0;
	_spatialPak->ampLDataSize = 0;
	_spatialPak->ampLGraphNum = 0;
	_spatialPak->ampLMin = 0;
	_spatialPak->ampLMax = 1;
	_spatialPak->ampLFLock = INACTIVE;

	_spatialPak->ampRView = NULL;
	_spatialPak->ampRArray = NULL;
	_spatialPak->ampRFile = NULL;
	_spatialPak->ampRStartRange = 0;
	_spatialPak->ampREndRange = 0;
	_spatialPak->ampRStartSel = 0;
	_spatialPak->ampREndSel = 0;
	_spatialPak->ampRXCurrent = 0;
	_spatialPak->ampRYCurrent = 0;
	_spatialPak->ampRFilePos = 0;
	_spatialPak->ampRArraySize = mixwaveResources->defaultTemplateSize();
	_spatialPak->ampRDataOffset = 0;
	_spatialPak->ampRDataSize = 0;
	_spatialPak->ampRGraphNum = 0;
	_spatialPak->ampRMin = 0;
	_spatialPak->ampRMax = 1;
	_spatialPak->ampRFLock = INACTIVE;

	_spatialPak->filterLType = 0;
	_spatialPak->filterRType = 0;

	_spatialPak->rvbTailTime = 0.0;
}
